"""Utility to create plots of CSV data in pure terminal environments"""

import pandas as pd
import plotille as plt


def plot_csv_data(filename, keys, delimiter=None):
    """Create a plot from a CSV file.

    The FILENAME is read with Pandas
    KEYS is a list of headers ["y/Y", "Cows", "Cats"]
    Plot KEYS[1:] with respect to KEYS[0] 

    Parameters:
    -----------
    :param filename: str, path to the csv file
    :param keys: list of str, names of the elements to plot
    :param delimiter: char, delimiter of the CSV file
    
    Returns :
    ---------
    graph: a string representing the graph
    """
    df_ = pd.read_csv(filename, delimiter=delimiter)

    try:
        fig = plt.Figure()
        fig.x_label=keys[0]

        x = df_[keys[0]].tolist() 
        for curve_key in keys[1:]:
            y = df_[curve_key].tolist() 

            fig.plot(x,y, label=curve_key)
    except KeyError as err:
        msg_err = f" : Possible keys are: {'|'.join(df_.keys())}"
        raise ValueError("Wrong columns names" + msg_err + str(err) )

    out = fig.show(legend=True)
    return out



"""interpolate a cloud from an other cloud 


Alternate version with Rbf for testing purpose.
NOT WORKING YET"""

import numpy as np
from scipy import spatial
from scipy.interpolate import Rbf

def RbfWeights(source, target, power, eps=1.e-16):
    def wrapper(d):
            shp = d.shape
            node_shp = shp[:-1]
            if d.ndim == 1:
                    mode = '1-D'
            else:
                    d = d.reshape(shp[0], np.prod(node_shp))
                    mode = 'N-D'
            function = lambda r: 1./(r+eps)**power
            return Rbf(*source, d, mode=mode, function=function)(*target).reshape(target[0].size, *node_shp)
    return wrapper

def cloud2cloud2(
    source_xyz, 
    source_val, 
    target_xyz, 
    power = 1.0):
    """ Interpolate  form a cloud to an other  NOT WORKING YET

    Parameters :
    ------------
    source_xyz : numpy array shape (:, 3) or (:,:, 3) or  (:,:,:, 3) 
    source_val : numpy array shape (:, 3) or (:,:, k) or  (:,:,:, k) k variables , first dims equal to source mesh.
    target_xyz : numpy array shape (:, 3) or (:,:, 3) or  (:,:,:, 3) 
    stencil (int): nb of neigbors to compute (1 is closest point)

    Optional keyword arguments
    --------------------------
    limitsource (int) : maximum nb of source points allowed (subsample beyond)
    power(float) : Description
    tol(float) : Description
    Returns :
    ----------
    target_val : numpy array shape (:, 3) or (:,:, 3) or  (:,:,:, 3) , first dims equal to target mesh.

    """

    def _extract_compnts(in_xyz):
        x = np.ravel(np.take(in_xyz, 0, axis=-1))
        y = np.ravel(np.take(in_xyz, 1, axis=-1))
        z = np.ravel(np.take(in_xyz, 2, axis=-1))
        return (x, y, z)
        

    interp_ = RbfWeights(
        _extract_compnts(source_xyz), 
        _extract_compnts(target_xyz),
        power = power
    )

    n_sce_pts = int(source_xyz.size/3)
    print("n_sce_pts", n_sce_pts )
    print("sce_val", source_val.shape )
    n_d_data = np.reshape(source_val, (n_sce_pts, -1))
    out_shape = list(target_xyz.shape)
    out_shape[-1] = source_val.shape[-1]
    n_d_interp_out = np.reshape(interp_(n_d_data), out_shape)

    return n_d_interp_out


###Testing a patch mapper 2D feature"""

import numpy as np
from arnica.utils import NpArray2Xmf

__all__ = ["AxiPointCloud"]

class AxiPointCloud():
    """Handle Axysymmetric points clouds as 1D meshs 
    with a dict of variables

    
    """

    def __init__(self,xcoor, ycoor, zcoor, name="Unnamed", vars=None, theta_range=None):
        """ startup class
        
        Parameters:
        ===========
        xcoor, ycoor, zcoor: coordinates 
            1D np array in meters
        name: name of the object
            string
        vars: variables associated to pointcloud
            dictionnary of 1D np.arrays
        theta_range: override the theta range, in radians
            float
        """

        self.name = name

        self.x = np.array(xcoor)
        self.y = np.array(ycoor)
        self.z = np.array(zcoor)
        if theta_range is None:
            self.recompute_theta_range_from_coords()
        else:
            self.theta_range=theta_range

        self.vars = dict()
        if vars is not None:
            for var in vars:
                self.vars[var] = np.array(vars[var])


    def __str__(self):
        """String representation of pt cloud"""

        def _print_data(name, arr):
            """utility to print arrays"""
            str_ = "-"+name+"----"
            str_ += "\nshape :"+ str(arr.shape)
            str_ += "\nmin/max :"+ str(arr.min())+"/"+str(arr.max())
            return str_

        str_ = f'\nAxiPointCloud object "{self.name}"'
        str_ += "\n\n Angle section (deg.):" + str(np.rad2deg(self.theta_range))
        str_ += "\n\n Coordinates:"
        str_ += "\n"+ _print_data("X coor (m)",self.x)
        str_ += "\n"+ _print_data("Y coor (m)",self.y)
        str_ += "\n"+ _print_data("Z coor (m)",self.z)
        str_ += "\n"+ _print_data("R coor (m)",self.rad())
        str_ += "\n"+ _print_data("Theta coor (deg)",np.rad2deg(self.theta()))
        str_ += "\n\n Data:"
        for key in self.vars:
            str_ += "\n"+ _print_data(key,self.vars[key])
        return str_

    def xyz(self):
        """Stacked version numpy of coordinates
        
        shape: (n, 3)"""
        return np.stack((self.x, self.y, self.z), axis=1)

    def vars_stack(self):
        """Stacked version numpy of variables
        
        shape: (n, k)"""

        keys = list(self.vars.keys())
        stack = self.vars[keys[0]]

        for key in keys[1:]:
            stack = np.stack((stack,self.vars[key] ), axis=1)
        return stack



    def rad(self):
        """Return radius np array from Y and Z"""
        return np.hypot(self.y,self.z)

    def theta(self):
        """Return theta np array (radians)
        
        - range -pi/pi
            - 0 on the y+ axis (z=0, y>0)
            spanning -pi to pi
                        0pi=0deg
                        Y
                        ^
                        |
                        |
        -0.5pi=-90deg     o------>Z   0.5pi=90deg
        """

        return np.arctan2(self.z,self.y)

    def recompute_theta_range_from_coords(self):
        """recompute theta range"""
        theta = self.theta()
        self.theta_range = theta.max() - theta.min()

    def rotate(self, shift_angle):
        """ rotate around x
        
        :param shift_angle: angle in radians
            float
        """

        radius = self.rad()
        theta = self.theta() + shift_angle

        self.y = radius * np.cos(theta)
        self.z = radius * np.sin(theta)


    def dupli_rotate(self, repeat=1):
        """ duplicate by rotation around x, in radians
        
        :param repeat: number or repetitions
            integer
        """

        radius = self.rad()
        theta = self.theta()
        for _ in range(repeat):
            theta += self.theta_range
            self.y = np.concatenate((self.y, radius * np.cos(theta)))
            self.z = np.concatenate((self.z, radius * np.sin(theta)))

        self.x = np.tile(self.x, repeat+1)
        for arr in self.vars:
            self.vars[arr] = np.tile(self.vars[arr], repeat+1)

        self.theta_range *= repeat+1

    def dump(self, filename):
        """Dump an XDMF file of the pointcloud"""

        stream = NpArray2Xmf(filename)
        stream.create_grid(self.x, self.y, self.z)
        for key in self.vars:
            stream.add_field(self.vars[key], key)
        stream.dump()



import numpy as np
import matplotlib
from matplotlib.ticker import FuncFormatter
import matplotlib.pyplot as plt
import scipy.stats as stats
import satis.temporal_analysis_tool as tat
import warnings


def resample_signal(time, signal, dtime=None):
    """
	*Resample the initial signal at a constant time interval.*
  
	:param time: Time vector of your signal
	:param signal: Signal vector
	:dtime: New time step
	:returns:            
	- **rescaled_time** - Uniformally rescaled time vector
	- **rescaled_signal** - Rescaled signal

	.. note:: 	If a dtime is given, the interpolation is made to 
				have a signal with a time interval of dtime.
				Else, the dt is the smallest time interval between
				two values of the signal.
	"""

    warnings.warn(("Prefer the satis.temporal_analysis_tool function"),
                  DeprecationWarning)
    rescaled_time, rescaled_signal = tat.resample_signal(time, signal, dtime)

    return rescaled_time, rescaled_signal


def calc_autocorrelation_time(time, signal, threshold=0.2):
    """
	*Estimate the autocorrelation time at a given threshold.*
	
	:param time: Time vector of your signal
	:param signal: Signal vector
	:param threshold: Threshold under which the signal is correlated
	:type threshold: float

	:returns:

	- **autocorrelation_time** - Minimum time step to capture the signal at a
								 correltion under thethreshold
	"""

    warnings.warn(("Prefer the satis.temporal_analysis_tool function"),
                  DeprecationWarning)

    autocorrelation_time = tat.calc_autocorrelation_time(time, signal, threshold)

    return autocorrelation_time


def show_autocorrelation_time(time, signal, threshold=0.2):
    """
	*Plot the autocorrelation function of the signal.*
	
	:param time: Time vector of your signal
	:param signal: Signal vector
	:param threshold: Autocorrelation threshold

	:returns:

	- **fig** - Figure of the result

	"""

    warnings.warn(("Prefer the satis.temporal_analysis_tool function"),
                  DeprecationWarning)

    figout = tat.show_autocorrelation_time(time, signal, threshold)

    return figout


def sort_spectral_power(time, signal):
    """
	*Determine the harmonic power contribution of the signal.*
	
	:param time: Time vector of your signal
	:param signal: Signal vector

	returns:

	- **harmonic_power** - Harmonic power of the signal
	- **total_power** - Total spectral power of the signal


	.. note:: 	It calculates the Power Spectral Density (PSD) of the complete
				signal and of a downsampled version of the signal. The 
				difference of the two PSD contains only harmonic components.
	"""

    warnings.warn(("Prefer the satis.temporal_analysis_tool function"),
                  DeprecationWarning)

    harmonic_power, total_power = tat.sort_spectral_power(time, signal)

    return harmonic_power, total_power


def power_representative_frequency(time, signal, threshold=0.8):
    """
	*Calculate the frequency that captures a level of spectral power.*

	:param time: Time vector of your signal
	:param signal: Signal vector
	:param threshold: Level of representativity of the spectral power

	:returns:

	- **representative_frequency** - Frequency above which the power threshold is reached

	.. note:: 	It calculates the cumulative power spectral density and returns
				the frequency that reaches the threshold of spectral power.
	"""

    warnings.warn(("Prefer the satis.temporal_analysis_tool function"),
                  DeprecationWarning)

    representative_frequency = tat.power_representative_frequency(time, signal, threshold)

    return representative_frequency


def show_power_representative_frequency(time, signal, threshold=0.80):
    """
	*Plot the power spectral density of the signal.*
	
	:param time: Time vector of your signal
	:param signal: Signal vector
	:param threshold: Power representative frequency threshold

	:returns:

	- **fig** - Figure of the result

	"""

    warnings.warn(("Prefer the satis.temporal_analysis_tool function"),
                  DeprecationWarning)

    figout = tat.show_power_representative_frequency(time, signal, threshold)

    return figout


def ks_test_distrib(data, distribution="normal"):
    """
	*Calculate the correlation score of the signal with the distribution*

	:param data: array of values
	:param distribution: kind of distribution the values follow to test

	:returns:

	- **score** -Minimum score over the height of the ks test
	- **position** -Index of the height at which the min. of the test is found
	- **height** -Corresponding heiht where the min. is found
	- **scale** -Scale parameter of the lognormal fitting

	"""
    warnings.warn(("Prefer the satis.temporal_analysis_tool function"),
                  DeprecationWarning)

    score, position, height, scale = tat.ks_test_distrib(data, distribution)

    return score, position, height, scale


def show_temperature_distribution(temperature_recording, height, distribution="normal"):
    """
	*Plot the temperature distribution and the fitting curve*

	:param temperature_recording: Temperature as a function of height and time
	:param height: Height in the plan40
	:param distribution: Type of distribution for the fitting method

	:returns:

	- **fig** - Figure of the result
	"""
    warnings.warn(("Prefer the satis.temporal_analysis_tool function"),
                  DeprecationWarning)

    figout = tat.show_temperature_distribution(temperature_recording, height, distribution)

    return figout


def duration_for_uncertainty(
    time, signal, target=10, confidence=0.95, distribution="normal"
):
    """
	*Give suggestion of simulation duration of a plan40 calculation.*
	
	:param time: Time vector of your signal
	:param signal: Signal vector
	:param target: Desired amplitude of the confidence interval
	:param confidence: Level of confidence of the interval
	:param distribution: Type of distribution of the signal to make the interval

	:returns:

	- **duration** -Duration of the signal
	"""

    warnings.warn(("Prefer the satis.temporal_analysis_tool function"),
                  DeprecationWarning)

    duration = tat.duration_for_uncertainty(time, signal, target, confidence, distribution)

    return duration


def uncertainty_from_duration(
    dtime, sigma, duration, confidence=0.95, distribution="normal"
):
    """
	*Give confidence interval length of a plan40 calculation.*

	:param dtime: Time step of your solutions
	:param sigma: Standard deviation of your signal
	:param duration: Desired duration of the signal
	:param confidence: Level of confidence of the interval
	:param distribution: Type of distribution of the signal to make the interval

	:returns:

	- **length** - Length of the confidence interval in K
	"""

    warnings.warn(("Prefer the satis.temporal_analysis_tool function"),
                  DeprecationWarning)

    length = tat.uncertainty_from_duration(dtime, sigma, duration, confidence, distribution)

    return length


def convergence_cartography(time, signal, **kwargs):
    """
	*Create a cartography of the convergence of the confidence interval in a simulation.*

	:param time: Time vector of your signal
	:param signal: Signal vector


	==**kwargs==
	
		:param max_time: Maximal simulation duration
		:param interlen: Maximal interval length

	:returns:
		- **fig** - Figure of the cartography

	"""
    warnings.warn(("Prefer the satis.temporal_analysis_tool function"),
                  DeprecationWarning)

    fig = tat.convergence_cartography(time, signal, **kwargs)

    return fig


def calculate_std(time, signal, frequency):
    """
	*Give the standard deviation of a signal at a given frequency.*
	
	:param time: Time vector of your signal
	:param signal: Signal vector
	:param frequency: Frequency at which values of the recording are taken

	:returns:

		- **std** - Standard deviation of the values taken from the recording
	"""

    warnings.warn(("Prefer the satis.temporal_analysis_tool function"),
                  DeprecationWarning)

    std = tat.calculate_std(time, signal, frequency)

    return std


def power_spectral_density(time, signal):
    """
	*Automate the computation of the Power Spectral Density of a signal.*

	:param time: Time vector of your signal
	:param signal: Signal vector

	:returns:

	- **frequency** -Frequency vector of the signal's power spectral density
	- **power_spectral_density** -Power spectral density of the signal
	"""

    warnings.warn(("Prefer the satis.temporal_analysis_tool function"),
                  DeprecationWarning)

    frequency, power_spectral_density = tat.power_spectral_density(time, signal)

    return frequency, power_spectral_density


def to_percent(y, position):
    """
	*Rescale the y-axis to per*
    
    """
    warnings.warn(("Prefer the satis.temporal_analysis_tool function"),
                  DeprecationWarning)

    s = tat.to_percent(y, position)

    return s 


def plot_distributions(path="./data.txt"):

    warnings.warn(("Prefer the satis.temporal_analysis_tool function"),
                  DeprecationWarning)

    tat.plot_distributions(path)


#!/usr/bin/env python
"""
cli.py

Command line interface for tools in Arnica
"""


import click


# --------------------------------------


@click.group()
def main_cli():
    """---------------    ARNICA  --------------------

You are now using the Command line interface of Arnica,
a Python3 helper for reactive multispecies computation, created at CERFACS (https://cerfacs.fr).

This is a python package currently installed in your python environement.
"""
    pass


@click.command()
@click.argument("filename", nargs=1)
@click.argument("list_vars", nargs=1)
def plotcsv(filename, list_vars):
    """Plot the content of a CSV file

    FILENAME is the path to the csv file
    LIST_VARS is the names of the column to plot separated by ":"
    the first will the the x_axis
    all the others will be the curves on y_axis
    
    ---

    Example:

    The CSV FILE should look like
    \n
    aa,bb,cc,dd\n
    1,2,3,4,5\n
    1,2,3,4,5\n
    1,2,3,4,5\n
    1,2,3,4,5\n

    The command
    >arnica plotcsv toto.csv aa:bb:cc:dd
    will plot

    bb .vs. aa, cc .vs. aa, dd .vs. aa, 


    """
    from arnica.utils.plotcsv import plot_csv_data

    print(plot_csv_data(filename, list_vars.split(":")))

main_cli.add_command(plotcsv)